#include "OpenSubtitles.h"
#include <QNetworkReply>
#include <fatrat/fatrat.h>
#include <fatrat/XmlRpc.h>
#include <zlib.h>
#include <fatrat/RuntimeException.h>
#include <fatrat/Settings.h>
#include <unistd.h>
#include <QFileDialog>
#include <QMessageBox>
#include <QtEndian>

#define USER_AGENT "FatRat"
#define SERVER_NAME "api.opensubtitles.org"
#define RPC_PATH "/xml-rpc"

OpenSubtitles::OpenSubtitles()
{
	m_network = new QNetworkAccessManager(this);
	connect(m_network, SIGNAL(finished(QNetworkReply*)), this, SLOT(requestFinished(QNetworkReply*)));
	connect(&m_timer, SIGNAL(timeout()), this, SLOT(noOperation()));
}

void OpenSubtitles::searchForMovie(QString file)
{
	qint64 filesize;
	QString checksum;
	
	checksum = computeMovieHash(file, filesize);
	if(!checksum.isEmpty())
		search(checksum, filesize);
}

void OpenSubtitles::search(QString checksum, qint64 fileSize)
{
	m_strChecksum = checksum;
	m_fileSize = fileSize;
	
//	treeResults->clear();
//	treeResults->setEnabled(false);
//	toolBrowse->setEnabled(false);
	
	if(m_strSession.isEmpty())
		createSession();
	else
		performSearch();
}

void OpenSubtitles::createSession()
{
	QByteArray postData;
	
	postData = XmlRpc::createCall(m_strLastFunction = "LogIn", QVariantList() << "" << "" << "eng" << USER_AGENT);
	QNetworkRequest req(QUrl("http://" SERVER_NAME RPC_PATH));
	req.setHeader(QNetworkRequest::ContentTypeHeader, "text/xml");
	req.setRawHeader("User-Agent", "FatRat");
	m_network->post(req, postData);
}

void OpenSubtitles::performSearch()
{
	QVariantList movies;
	QByteArray postData;
	
	QStringList langs = getSettingsValue("subtitle_search/languages").toString().split(';', QString::SkipEmptyParts);
	
	foreach(QString lang, langs)
	{
		QVariantMap movie;
		movie["sublanguageid"] = lang;
		movie["moviehash"] = m_strChecksum;
		movie["moviebytesize"] = double(m_fileSize);
		movies << movie;
	}
	
	postData = XmlRpc::createCall(m_strLastFunction = "SearchSubtitles", QVariantList() << m_strSession << QVariant(movies));
	QNetworkRequest req(QUrl("http://" SERVER_NAME RPC_PATH));
	req.setHeader(QNetworkRequest::ContentTypeHeader, "text/xml");
	req.setRawHeader("User-Agent", "FatRat");
	m_network->post(req, postData);
}

void OpenSubtitles::noOperation()
{
	QByteArray postData;
	
	postData = XmlRpc::createCall(m_strLastFunction = "NoOperation", QVariantList() << m_strSession);
	
	QNetworkRequest req(QUrl("http://" SERVER_NAME RPC_PATH));
	req.setHeader(QNetworkRequest::ContentTypeHeader, "text/xml");
	req.setRawHeader("User-Agent", "FatRat");
	m_network->post(req, postData);
}

QString OpenSubtitles::computeMovieHash(QString filename, qint64& fsize)
{
	const int CHUNK = 65536;
	qint64* buffer;
	QFile file(filename);
	int read;
	quint64 checksum;
	
	if(!file.open(QIODevice::ReadOnly))
	{
		qDebug() << "Failed to open" << filename;
		return QString();
	}
	
	buffer = new qint64[CHUNK/8];
	checksum = fsize = file.size();
	
	read = file.read((char*) buffer, CHUNK);
	for(int i=0;i<read/8;i++)
		checksum += qFromLittleEndian(buffer[i]);
	
	file.seek(qMax<qint64>(0, fsize-CHUNK));
	
	read = file.read((char*) buffer, CHUNK);
	for(int i=0;i<read/8;i++)
		checksum += qFromLittleEndian(buffer[i]);
	
	delete [] buffer;
	
	if(fsize < 65536)
	{
		QMessageBox::warning(getMainWindow(), "FatRat",
			QObject::tr("You have selected a file smaller than 64 kilobytes. "
				"The OpenSubtitles.org checksumming algorithm is not designed to deal with such files. "
				"Therefore the generated checksum is very likely unusable."));
	}
	
	return QString::number(checksum, 16);
}

void OpenSubtitles::downloadMovie(QString fileName, QString id)
{
	QByteArray postData;
	QVariantList subs;
	subs << id;
	postData = XmlRpc::createCall(m_strLastFunction = "DownloadSubtitles", QVariantList() << m_strSession << QVariant(subs));
	
	m_strFileName = fileName;
	
	QNetworkRequest req(QUrl("http://" SERVER_NAME RPC_PATH));
	req.setHeader(QNetworkRequest::ContentTypeHeader, "text/xml");
	req.setRawHeader("User-Agent", "FatRat");
	m_network->post(req, postData);
}

void OpenSubtitles::requestFinished(QNetworkReply* reply)
{
	reply->deleteLater();
	
	if (reply->error() != QNetworkReply::NoError)
	{
		emit error(reply->errorString());
		return;
	}
	
	try
	{
		QVariant result = XmlRpc::parseResponse(reply->readAll());
		if(result.isNull())
			throw RuntimeException(tr("The server has returned an empty result"));
		
		if(m_strLastFunction == "LogIn")
		{
			QByteArray postData;
			
			m_strSession = result.toMap()["token"].toString();
			
			m_timer.start(10*60*1000);
			connect(&m_timer, SIGNAL(timeout()), this, SLOT(noOperation()));

			if(!m_strChecksum.isEmpty())
				performSearch();
//			else
//			{
//				treeResults->setEnabled(true);
//				toolBrowse->setEnabled(true);
//			}
		}
		else if(m_strLastFunction == "SearchSubtitles")
		{
			QVariantList list = result.toMap()["data"].toList();
			QList<SubInfo> rv;
			
			foreach(QVariant sub, list)
			{
				QVariantMap map = sub.toMap();
				SubInfo info;
				
				info.movieName = map["MovieName"].toString();
				info.releaseName = map["MovieReleaseName"].toString();
				info.language = map["LanguageName"].toString();
				info.year = map["MovieYear"].toInt();
				info.langCode = map["SubLanguageID"].toString();
				info.id = map["IDSubtitleFile"].toInt();
				info.subFormat = map["SubFormat"].toString().toLower();
				info.rating = map["SubRating"].toString();
				info.downloads = map["SubDownloadsCnt"].toInt();
				
				int cds = map["SubSumCD"].toInt();
				if(cds)
					info.discinfo = QString("%1/%2").arg(map["SubActualCD"].toInt()).arg(cds);
				
				rv << info;
			}
			
			//if(list.isEmpty())
			//	QMessageBox::information(this, "FatRat", tr("No subtitles found!"));
			emit moviesFound(rv);
			
			m_strChecksum.clear();
		}
		else if(m_strLastFunction == "DownloadSubtitles")
		{
			QString proposedName = m_strFileName;
			proposedName = QFileDialog::getSaveFileName(getMainWindow(), "FatRat", proposedName);
			
			if(!proposedName.isEmpty())
			{
				int fds[2];
				
				if(!::pipe(fds))
				{
					// The server sends it as a string, despite the fact that it's a base64
					// encoded data, for which the XML-RPC specifies a distinct type
					// Therefore we have to do the base64 decoding now
					
					QVariantList list = result.toMap()["data"].toList();
					QByteArray data;
					
					if(list.isEmpty())
						throw RuntimeException(tr("The server didn't return the requested data"));
					
					data = QByteArray::fromBase64(list[0].toMap()["data"].toByteArray());
					
					::write(fds[1], data.constData(), data.size());
					::close(fds[1]);
					
					data.clear();
					
					decompressFile(fds[0], proposedName);
					emit downloadDone();
				}
				else
					throw RuntimeException(tr("Failed to allocate a pipe"));
			}
		}
	}
	catch(const RuntimeException& e)
	{
		//QMessageBox::critical(this, "FatRat", e.what());
		//toolBrowse->setEnabled(true);
		emit error(e.what());
	}
}

void OpenSubtitles::decompressFile(int infd, QString unzipped)
{
	char buffer[4096];
	QFile fout(unzipped);
	gzFile file;
	
	if(!fout.open(QIODevice::WriteOnly))
		return;
	
	file = gzdopen(infd, "rb");
	if(!file)
		return;
	
	while(true)
	{
		int bytes = gzread(file, buffer, sizeof buffer);
		if(bytes <= 0)
			break;
		fout.write(buffer, bytes);
	}
	
	gzclose(file);
}

