/*
FatRat download manager
http://fatrat.dolezel.info

Copyright (C) 2006-2008 Lubos Dolezel <lubos a dolezel.info>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
version 2 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "SubtitlesDlg.h"

#include <fatrat/fatrat.h>
#include <fatrat/RuntimeException.h>
#include <fatrat/Settings.h>

#include <unistd.h>

#include <QMessageBox>
#include <QFileDialog>
#include <QHeaderView>
#include <QBuffer>
#include <QHttp>
#include <QFile>
#include <QtDebug>

#include "OpenSubtitles.h"
#include "Sublight.h"

extern const char* g_movieSuffixes[];

SubtitlesDlg::SubtitlesDlg(QWidget* parent)
	: QDialog(parent)
{
	setupUi(this);
	
	qRegisterMetaType<QList<SubInfo> >("QList<SubInfo>");
	
	QStringList hdr = QStringList() << tr("Name") << tr("Language") << tr("Release name") << tr("Part") << tr("Downloads") << tr("Rating");
	treeResults->setHeaderLabels(hdr);
	
	QHeaderView* phdr = treeResults->header();
	phdr->resizeSection(0, 200);
	phdr->resizeSection(1, 80);
	phdr->resizeSection(3, 60);
	phdr->resizeSection(4, 80);
	phdr->resizeSection(5, 80);
	
	connect(toolBrowse, SIGNAL(clicked()), this, SLOT(chooseFile()));
	connect(treeResults, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), this, SLOT(itemDoubleClicked(QTreeWidgetItem*)));
	connect(this, SIGNAL(finished(int)), this, SLOT(deleteLater()));
	
	if (getSettingsValue("subtitle_search/opensubtitles", true).toBool())
		m_engines << new OpenSubtitles;
	if (getSettingsValue("subtitle_search/sublight", true).toBool())
		m_engines << new Sublight;
	
	foreach (SearchEngine* e, m_engines)
	{
		connect(e, SIGNAL(moviesFound(QList<SubInfo>)), this, SLOT(moviesFound(QList<SubInfo>)));
		connect(e, SIGNAL(error(QString)), this, SLOT(error(QString)));
		connect(e, SIGNAL(downloadDone()), this, SLOT(downloadDone()));
	}
	if (m_engines.isEmpty())
	{
		QMessageBox::critical(this, "FatRat", tr("No search engines enabled."));
		hide();
	}
}

SubtitlesDlg::~SubtitlesDlg()
{
	qDeleteAll(m_engines);
}

QWidget* SubtitlesDlg::create()
{
	return new SubtitlesDlg(getMainWindow());
}

void SubtitlesDlg::chooseFile()
{
	QString filter = "(";
	QString chosenFile;
	
	for(size_t i=0;g_movieSuffixes[i];i++)
	{
		filter += '*';
		filter += g_movieSuffixes[i];
		filter += ' ';
	}
	filter += ')';
	
	chosenFile = QFileDialog::getOpenFileName(this, "FatRat", lineFile->text(), filter);
	
	if(chosenFile.isEmpty())
		return;
	
	search(chosenFile);
}

void SubtitlesDlg::search(QString file)
{
	lineFile->setText(file);

	treeResults->clear();
	treeResults->setEnabled(false);
	toolBrowse->setEnabled(false);
	
	m_numSearching = m_engines.size();
	foreach (SearchEngine* e, m_engines)
		e->searchForMovie(file);
}

void SubtitlesDlg::moviesFound(QList<SubInfo> movies)
{
	foreach(const SubInfo& sub, movies)
	{
		SubtitleTreeWidgetItem* item = new SubtitleTreeWidgetItem(treeResults);
		item->setText(0, QString("%1 (%2)").arg(sub.movieName).arg(sub.year));
		item->setText(1, sub.language);
		item->setText(2, sub.releaseName);

		item->setText(3, sub.discinfo);
		
		item->setText(4, QString::number(sub.downloads));
		item->setText(5, sub.rating);

		item->m_engine = static_cast<SearchEngine*>(sender());
		item->m_id = sub.id;
		item->m_lang = sub.langCode;
		item->m_format = sub.subFormat;

		treeResults->addTopLevelItem(item);
	}
	
	if (!--m_numSearching)
	{
		treeResults->setEnabled(true);
		toolBrowse->setEnabled(true);
		
		if (!treeResults->topLevelItemCount())
			QMessageBox::warning(this, "FatRat", tr("No subtitles found!"));
	}
}

void SubtitlesDlg::error(QString err)
{
	QMessageBox::warning(this, "FatRat", err);
	
	if (!--m_numSearching)
	{
		treeResults->setEnabled(true);
		toolBrowse->setEnabled(true);
	}
}

void SubtitlesDlg::downloadDone()
{
	treeResults->setEnabled(true);
	toolBrowse->setEnabled(true);
}

void SubtitlesDlg::itemDoubleClicked(QTreeWidgetItem* i)
{
	SubtitleTreeWidgetItem* item = static_cast<SubtitleTreeWidgetItem*>(i);
	
	QString proposedName, originalName, filter;
	originalName = lineFile->text();
	
	int pos = originalName.lastIndexOf('.');
	if(pos < 0)
		pos = originalName.size() - 4;
	
	proposedName = originalName.left(pos+1);
	proposedName += QString("%1.%2").arg(item->m_lang).arg(item->m_format);
	
	treeResults->setEnabled(false);
	toolBrowse->setEnabled(false);
	
	item->m_engine->downloadMovie(proposedName, item->m_id);
}

